# pylint: disable=missing-module-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
from enum import Enum, auto

class AutoStrEnum(str, Enum):
    @staticmethod
    def _generate_next_value_(name, start, count, last_values):
        return name.lower()

    def __str__(self) -> str:
        return self.value

class LogLevel(Enum):
    NOTSET=0
    DEBUG =10
    INFO = 20
    NOTICE= 25
    WARNING=30
    ERROR=40
    CRITICAL=50

    def __str__(self):
        return self.name

class LoggingHandler(AutoStrEnum):

    """
    Standardized remote logging handlers for data engineering pipelines,
    designed for easy analysis and identification of remote logging
    requirements
    """
    
    NONE = auto()  # No remote handler
    LOCAL_STREAM = auto()  # Local stream handler
    GCP_CLOUD_LOGGING = auto()
    GCP_ERROR_REPORTING = auto()
    GCP_FIREBASE = auto()
    AWS_CLOUD_WATCH = auto()
    AZURE_MONITOR = auto()
    AZURE_APPLICATION_INSIGHTS = auto()
    IBM_LOG_ANALYTICS = auto()
    ALIBABA_LOG_SERVICE = auto()
    LOGGLY = auto()
    DATADOG = auto()
    NEW_RELIC = auto()
    SENTRY = auto()
    SUMOLOGIC = auto()
    # --- Other ---
    SYSLOG = auto() # For system logs
    CUSTOM = auto() # For a user-defined remote handler
    OTHER = auto()

    def __str__(self):
        return self.name


class LogLevelPro(Enum):
    """
    Standardized notice levels for data engineering pipelines,
    designed for easy analysis and identification of manual 
    intervention needs.
    """
    ##############################
    # DEBUG (1000 - 9990) --> LOGGED AT DEBUG LEVEL
    ##############################
    DEBUG = 1000

    ##############################
    # INFO (10000 - 19990)
    ##############################
    INFO = 10000
    # Task Info (11100 - 11190)
    INFO_TASK_STARTED = 11100
    INFO_TASK_COMPLETE = 11110
    INFO_TASK_COMPLETE_WITH_NOTICES = 11120
    INFO_TASK_COMPLETE_WITH_WARNINGS = 11130
    INFO_TASK_PAUSED = 11140
    INFO_TASK_RESUMED = 11150
    INFO_TASK_CANCELLED = 11160
    INFO_TASK_STOPPED = 11170
    INFO_TASK_SKIPPED = 11180

    # Subject Info (12100 - 12190)
    INFO_SUBJECT_STARTED = 12100

    # Subject Group Info (12200 - 12290)
    INFO_SUBJECT_GROUP_STARTED = 12200

    # Iteration Info (13100 - 13190)
    INFO_ITERATION_STARTED = 13100
    INFO_ITERATION_PAUSED = 13110
    INFO_ITERATION_RESUMED = 13120
    INFO_ITERATION_CANCELLED = 13130
    INFO_ITERATION_STOPPED = 13140
    INFO_ITERATION_SKIPPED = 13150

    # Pipeline Info (14100 - 14190)
    INFO_PIPELINE_STARTED = 14100
    INFO_PIPELINE_PAUSED = 14110
    INFO_PIPELINE_RESUMED = 14120
    INFO_PIPELINE_CANCELLED = 14130
    INFO_PIPELINE_STOPPED = 14140

    ##############################
    # READS (20000 - 29990)
    ##############################
    READ = 20000
    READ_COMPLETE = 20010
    READ_DB_COMPLETE = 20020
    READ_DOCUMENT_COMPLETE = 20030
    READ_FILE_FROM_CLOUD_STORAGE_COMPLETE = 20040
    READ_FILE_FROM_LOCAL_STORAGE_COMPLETE = 20050
    READ_HTTP_GET_COMPLETE = 20060
    READ_SECRET_FROM_SECRET_MANAGER_COMPLETE = 20070

    # PubSub Reads (20200 - 20290)
    READ_PUBSUB_PULL_MESSAGE_COMPLETE = 20200
    READ_PUBSUB_PUSH_MESSAGE_COMPLETE = 20210
    READ_PUBSUB_LIST_SUBSCRIPTIONS_COMPLETE = 20220
    READ_PUBSUB_LIST_TOPICS_COMPLETE = 20230
    READ_PUBSUB_LIST_SUBSCRIPTIONS_FOR_TOPIC_COMPLETE = 20240

    ##############################
    # IN-MEMORY TASKS (30000 - 39990)
    ##############################
    IN_MEMORY_TASK = 30000
    # Validation (30100 - 30990)
    IN_MEMORY_VALIDATE_SCHEMA_COMPLETE = 30100
    IN_MEMORY_VALIDATE_COMPLETE = 30110

    # Sorting/Grouping/Joining (31000 - 31990)
    IN_MEMORY_SORT_COMPLETE = 31000
    IN_MEMORY_GROUP_BY_COMPLETE = 31010
    IN_MEMORY_JOIN_COMPLETE = 31020
    IN_MEMORY_MERGE_COMPLETE = 31030
    IN_MEMORY_SPLIT_COMPLETE = 31040
    IN_MEMORY_UNION_COMPLETE = 31050

    # Transformation (32000 - 32990)
    IN_MEMORY_TRANSFORM_COMPLETE = 32000
    IN_MEMORY_FILTER_COMPLETE = 32010
    IN_MEMORY_AGGREGATE_COMPLETE = 32020
    IN_MEMORY_NORMALIZE_COMPLETE = 32030
    IN_MEMORY_DENORMALIZE_COMPLETE = 32040
    IN_MEMORY_DEDUPLICATE_COMPLETE = 32050
    IN_MEMORY_ENRICH_COMPLETE = 32060
    IN_MEMORY_DECRYPT_COMPLETE = 32070
    IN_MEMORY_ENCRYPT_COMPLETE = 32080
    IN_MEMORY_MASK_COMPLETE = 32090
    IN_MEMORY_FILL_MISSING_VALUES_COMPLETE = 32100
    IN_MEMORY_REPLACE_VALUES_COMPLETE = 32110

    ##############################
    # PERSIST (40000 - 49990) --> ALSO LOGGED AT INFO LEVEL
    ##############################
    PERSIST = 40000  # General PERSIST, no immediate PERSIST required

    # General PERSIST Completions (41000 - 41990)
    PERSIST_COMPLETE = 41000  # General PERSIST, no immediate PERSIST required
    PERSIST_CREATE_COMPLETE = 41010
    PERSIST_WRITE_COMPLETE = 41020
    PERSIST_UPDATE_COMPLETE = 41030
    PERSIST_DELETE_COMPLETE = 41040
    PERSIST_MERGE_COMPLETE = 41050

    # BATCH PERSIST Completions (42000 - 42990)
    PERSIST_BATCH_COMPLETE = 42000
    PERSIST_CREATE_BATCH_COMPLETE = 42010
    PERSIST_WRITE_BATCH_COMPLETE = 42020
    PERSIST_UPDATE_BATCH_COMPLETE = 42030
    PERSIST_DELETE_BATCH_COMPLETE = 42040

    # Cloud Storage PERSISTs (43000 - 43990)
    PERSIST_WRITE_IN_CLOUD_STORAGE_COMPLETE = 43000
    PERSIST_UPDATE_IN_CLOUD_STORAGE_COMPLETE = 43010
    PERSIST_DELETE_IN_CLOUD_STORAGE_COMPLETE = 43020
    PERSIST_WRITE_BATCH_IN_CLOUD_STORAGE_COMPLETE = 43100
    PERSIST_UPDATE_BATCH_IN_CLOUD_STORAGE_COMPLETE = 43110
    PERSIST_DELETE_BATCH_IN_CLOUD_STORAGE_COMPLETE = 43120

    # Local Storage PERSISTs (44000 - 44990)
    PERSIST_WRITE_IN_LOCAL_STORAGE_COMPLETE = 44000
    PERSIST_UPDATE_IN_LOCAL_STORAGE_COMPLETE = 44010
    PERSIST_DELETE_IN_LOCAL_STORAGE_COMPLETE = 44020
    PERSIST_WRITE_BATCH_IN_LOCAL_STORAGE_COMPLETE = 44100
    PERSIST_UPDATE_BATCH_IN_LOCAL_STORAGE_COMPLETE = 44110
    PERSIST_DELETE_BATCH_IN_LOCAL_STORAGE_COMPLETE = 44120

    # Cloud DB PERSISTs (45000 - 45990)
    PERSIST_WRITE_IN_DB_COMPLETE = 45000
    PERSIST_UPDATE_IN_DB_COMPLETE = 45010
    PERSIST_DELETE_IN_DB_COMPLETE = 45020
    PERSIST_MERGE_IN_DB_COMPLETE = 45030

    # BATCH Cloud DB PERSISTs (45100 - 45990)
    PERSIST_WRITE_BATCH_IN_DB_COMPLETE = 45100
    PERSIST_UPDATE_BATCH_IN_DB_COMPLETE = 45110
    PERSIST_DELETE_BATCH_IN_DB_COMPLETE = 45120
    PERSIST_MERGE_BATCH_IN_DB_COMPLETE = 45130

    # Cloud Create PERSISTs (45500 - 45690)
    PERSIST_CREATE_DB_TABLE_COMPLETE = 45500
    PERSIST_CREATE_OR_REPLACE_DB_TABLE_COMPLETE = 45510
    PERSIST_CREATE_AND_LOAD_DB_TABLE_COMPLETE = 45520
    PERSIST_DELETE_DB_TABLE_COMPLETE = 45530

    # Cloud Collection PERSISTs (45700 - 45990)
    PERSIST_CREATE_DB_COLLECTION_COMPLETE = 45700
    PERSIST_DELETE_DB_COLLECTION_COMPLETE = 45710

    # HTTP PERSISTs (47000 - 47990)
    PERSIST_WRITE_HTTP_POST_COMPLETE = 47000
    PERSIST_WRITE_HTTP_PUT_COMPLETE = 47010
    PERSIST_WRITE_HTTP_PATCH_COMPLETE = 47020
    PERSIST_WRITE_HTTP_DELETE_COMPLETE = 47030

    # PubSub PERSISTs (48000 - 48990)
    PERSIST_PUBSUB_PUBLISH_MESSAGE_COMPLETE = 48000
    PERSIST_PUBSUB_ACK_MESSAGE_COMPLETE = 48010
    PERSIST_PUBSUB_CREATE_TOPIC_COMPLETE = 48100
    PERSIST_PUBSUB_DELETE_TOPIC_COMPLETE = 48110
    PERSIST_PUBSUB_CREATE_SUBSCRIPTION_COMPLETE = 48200
    PERSIST_PUBSUB_DELETE_SUBSCRIPTION_COMPLETE = 48210


    ##############################
    # SUCCESSES (50000 - 54990) --> LOGGED AT INFO LEVEL
    ##############################
    SUCCESS = 50000
    # # Task Successes (5100 - 5199) ---> TASK SUCCESSES ARE INFO LEVEL, HENCE ARE IN INFO
    # Subject Successes (52000 - 52990)
    SUCCESS_SUBJECT_COMPLETE = 52000
    SUCCESS_SUBJECT_COMPLETE_WITH_NOTICES = 52010
    SUCCESS_SUBJECT_COMPLETE_WITH_WARNINGS = 52020

    SUCCESS_SUBJECT_GROUP_COMPLETE = 52030
    SUCCESS_SUBJECT_GROUP_COMPLETE_WITH_NOTICES = 52040
    SUCCESS_SUBJECT_GROUP_COMPLETE_WITH_WARNINGS = 52050

    # Iteration Successes (53000 - 53990)
    SUCCESS_ITERATION_COMPLETE = 53000
    SUCCESS_ITERATION_COMPLETE_WITH_NOTICES = 53010
    SUCCESS_ITERATION_COMPLETE_WITH_WARNINGS = 53020

    # Pipeline Successes (54000 - 54990)
    SUCCESS_PIPELINE_COMPLETE = 54000
    SUCCESS_PIPELINE_COMPLETE_WITH_NOTICES = 54010
    SUCCESS_PIPELINE_COMPLETE_WITH_WARNINGS = 54020

    ##############################
    # NOTICES (60000 - 61990) --> LOGGED AT INFO LEVEL
    ##############################
    NOTICE = 60000

    NOTICE_ALREADY_EXISTS = 61000
    NOTICE_PARTIAL_EXISTS = 61010

    NOTICE_FILE_IN_CLOUD_STORAGE_ALREADY_EXISTS = 61100
    NOTICE_DATA_IN_DB_ALREADY_EXISTS = 61110
    NOTICE_DATA_IN_DB_PARTIALLY_EXISTS = 61120
    NOTICE_TABLE_DOESNT_EXIST = 61130
    NOTICE_COLLECTION_DOESNT_EXIST = 61140
    NOTICE_DOCUMENT_DOESNT_EXIST = 61150

    ##############################
    # WARNINGS (70000 - 72990) --> LOGGED AT WARNING LEVEL
    ##############################
    WARNING = 70000

    WARNING_REVIEW_RECOMMENDED = 71000
    WARNING_DATA_SCHEMA_ISSUE = 72000
    WARNING_METADATA_SCHEMA_ISSUE = 72010
    WARNING_VALIDATION_ISSUES = 72020

    ##############################
    # ERRORS (80000 - 89990) --> LOGGED AT ERROR LEVEL
    ##############################
    ERROR = 80000
    ERROR_EXCEPTION = 80010
    ERROR_CUSTOM = 80020

    ERROR_DATA_QUALITY_ISSUE = 80030
    ERROR_VALIDATION_ISSUE = 80040
    ERROR_SCHEMA_ISSUE = 80050
    ERROR_SCHEMA_VALIDATION_ISSUE = 80060

    # Read Errors (82000 - 82990)
    ERROR_READ = 82010
    ERROR_READ_DB = 82100

    ERROR_READ_FILE_FROM_CLOUD_STORAGE = 82300
    ERROR_READ_FILE_FROM_LOCAL_STORAGE = 82310
    ERROR_READ_FILE_FROM_CLOUD_STORAGE_WITH_ERRORS = 82320
    ERROR_READ_FILE_FROM_LOCAL_STORAGE_WITH_ERRORS = 82330

    ERROR_READ_HTTP_GET = 82500
    ERROR_READ_SECRET_FROM_SECRET_MANAGER = 82600

    ERROR_READ_PUBSUB_PULL = 82710
    ERROR_READ_PUBSUB_PUSH = 82720

    # In-Memory Task Errors (83000 - 83990)
    ERROR_IN_MEMORY_TASK = 83100
    ERROR_IN_MEMORY_TRANSFORMATION = 83120

    ERROR_IN_MEMORY_VALIDATE = 83510
    ERROR_IN_MEMORY_VALIDATE_SCHEMA = 83520

    # PERSIST Errors (84000 - 84990)
    ERROR_PERSIST_PARTIALLY_FAILED = 84000
    ERROR_PERSIST_FAILED = 84010
    ERROR_PERSIST_WITH_ERRORS = 84020
    ERROR_PERSIST_WITH_WARNINGS_OR_ERRORS = 84030

    # General Data Persistence Errors (84100 - 84290)
    ERROR_PERSIST_CREATE = 84210
    ERROR_PERSIST_WRITE = 84220
    ERROR_PERSIST_UPDATE = 84230
    ERROR_PERSIST_DELETE = 84240
    ERROR_PERSIST_MERGE = 84250

    # Data Persistence Errors with Warnings (84400 - 84490)
    ERROR_PERSIST_CREATE_WITH_ERRORS = 84410
    ERROR_PERSIST_WRITE_WITH_ERRORS = 84420
    ERROR_PERSIST_UPDATE_WITH_ERRORS = 84430
    ERROR_PERSIST_DELETE_WITH_ERRORS = 84440
    ERROR_PERSIST_MERGE_WITH_ERRORS = 84450

    # Cloud Storage Errors (84500 - 84590)
    ERROR_PERSIST_WRITE_IN_CLOUD_STORAGE_FAILED = 84500
    ERROR_PERSIST_UPDATE_IN_CLOUD_STORAGE_FAILED = 84510
    ERROR_PERSIST_DELETE_IN_CLOUD_STORAGE_FAILED = 84520
    ERROR_PERSIST_MERGE_IN_CLOUD_STORAGE_FAILED = 84530

    # Cloud Storage Errors with Warnings (84600 - 84690)
    ERROR_PERSIST_WRITE_IN_CLOUD_STORAGE_WITH_ERRORS = 84550
    ERROR_PERSIST_UPDATE_IN_CLOUD_STORAGE_WITH_ERRORS = 84560
    ERROR_PERSIST_DELETE_IN_CLOUD_STORAGE_WITH_ERRORS = 84580
    ERROR_PERSIST_MERGE_IN_CLOUD_STORAGE_WITH_ERRORS = 84590

    # Local Storage Persistence Errors (84600 - 84690)
    ERROR_PERSIST_WRITE_IN_LOCAL_STORAGE_FAILED = 84600
    ERROR_PERSIST_UPDATE_IN_LOCAL_STORAGE_FAILED = 84610
    ERROR_PERSIST_DELETE_IN_LOCAL_STORAGE_FAILED = 84620
    ERROR_PERSIST_MERGE_IN_LOCAL_STORAGE_FAILED = 84630

    # Local Storage Errors with Warnings (84700 - 84790)
    ERROR_PERSIST_WRITE_IN_LOCAL_STORAGE_WITH_ERRORS = 84650
    ERROR_PERSIST_UPDATE_IN_LOCAL_STORAGE_WITH_ERRORS = 84660
    ERROR_PERSIST_DELETE_IN_LOCAL_STORAGE_WITH_ERRORS = 84670
    ERROR_PERSIST_MERGE_IN_LOCAL_STORAGE_WITH_ERRORS = 84680

    # Database Persistence Errors (84800 - 84890)
    ERROR_PERSIST_WRITE_IN_DB_FAILED = 84800
    ERROR_PERSIST_UPDATE_IN_DB_FAILED = 84810
    ERROR_PERSIST_DELETE_IN_DB_FAILED = 84820
    ERROR_PERSIST_MERGE_IN_DB_FAILED = 84830

    # Database Errors with Warnings (84850 - 84890)
    ERROR_PERSIST_WRITE_IN_DB_WITH_ERRORS = 84850
    ERROR_PERSIST_UPDATE_IN_DB_WITH_ERRORS = 84860
    ERROR_PERSIST_DELETE_IN_DB_WITH_ERRORS = 84870
    ERROR_PERSIST_MERGE_IN_DB_WITH_ERRORS = 84880

    # Cloud Create Persistence Errors (86000 - 86290)
    ERROR_PERSIST_CREATE_DB_TABLE_FAILED = 86000
    ERROR_PERSIST_CREATE_OR_REPLACE_DB_TABLE_FAILED = 86010
    ERROR_PERSIST_CREATE_AND_LOAD_DB_TABLE_FAILED = 86020
    ERROR_PERSIST_DELETE_DB_TABLE_FAILED = 86030

    ERROR_PERSIST_CREATE_DB_TABLE_WITH_ERRORS = 86200
    ERROR_PERSIST_CREATE_OR_REPLACE_DB_TABLE_WITH_ERRORS = 86210
    ERROR_PERSIST_CREATE_AND_LOAD_DB_TABLE_WITH_ERRORS = 86220
    ERROR_PERSIST_DELETE_DB_TABLE_WITH_ERRORS = 86230

    # Cloud Collection Persistence Errors (86400 - 86590)
    ERROR_PERSIST_CREATE_DB_COLLECTION_FAILED = 86400
    ERROR_PERSIST_DELETE_DB_COLLECTION_FAILED = 86410

    ERROR_PERSIST_CREATE_DB_COLLECTION_WITH_ERRORS = 86500
    ERROR_PERSIST_DELETE_DB_COLLECTION_WITH_ERRORS = 86510

    # HTTP Persistence Errors (87000 - 87299)
    ERROR_PERSIST_WRITE_HTTP_POST_FAILED = 87000
    ERROR_PERSIST_WRITE_HTTP_PUT_FAILED = 87010
    ERROR_PERSIST_WRITE_HTTP_PATCH_FAILED = 87020
    ERROR_PERSIST_WRITE_HTTP_DELETE_FAILED = 87030

    ERROR_PERSIST_WRITE_HTTP_POST_WITH_ERRORS = 87200
    ERROR_PERSIST_WRITE_HTTP_PUT_WITH_ERRORS = 87210
    ERROR_PERSIST_WRITE_HTTP_PATCH_WITH_ERRORS = 87220
    ERROR_PERSIST_WRITE_HTTP_DELETE_WITH_ERRORS = 87230

    # PubSub Persistence Errors (87500 - 87799)
    ERROR_PERSIST_PUBSUB_PUBLISH_MESSAGE_FAILED = 87500
    ERROR_PERSIST_PUBSUB_ACK_MESSAGE_FAILED = 87510
    ERROR_PERSIST_PUBSUB_CREATE_TOPIC_FAILED = 87520
    ERROR_PERSIST_PUBSUB_DELETE_TOPIC_FAILED = 87530
    ERROR_PERSIST_PUBSUB_CREATE_SUBSCRIPTION_FAILED = 87540
    ERROR_PERSIST_PUBSUB_DELETE_SUBSCRIPTION_FAILED = 87550

    ERROR_PERSIST_PUBSUB_PUBLISH_MESSAGE_WITH_ERRORS = 87700
    ERROR_PERSIST_PUBSUB_ACK_MESSAGE_WITH_ERRORS = 87710
    ERROR_PERSIST_PUBSUB_CREATE_TOPIC_WITH_ERRORS = 87720
    ERROR_PERSIST_PUBSUB_DELETE_TOPIC_WITH_ERRORS = 87730
    ERROR_PERSIST_PUBSUB_CREATE_SUBSCRIPTION_WITH_ERRORS = 87740
    ERROR_PERSIST_PUBSUB_DELETE_SUBSCRIPTION_WITH_ERRORS = 87750

    # Threshold Errors (88000 - 89999)
    ERROR_TASK_ALLOWED_THRESHOLD_REACHED = 88000
    ERROR_DATA_QUALITY_ALLOWED_THRESHOLD_REACHED = 88010
    ERROR_METADATA_QUALITY_ALLOWED_THRESHOLD_REACHED = 88020

    ERROR_SUBJECT_ALLOWED_THRESHOLD_REACHED = 88110
    ERROR_ITERATION_ALLOWED_THRESHOLD_REACHED = 88120
    ERROR_PIPELINE_ALLOWED_THRESHOLD_REACHED = 88130

    ##############################
    # FAILURES (90000 - 99990) --> LOGGED AT ERROR LEVEL
    ##############################
    FAILED = 90000

    # Task Failures (91000 - 91990)
    FAILED_TASK = 91000
    FAILED_TASK_COMPLETE_WITH_ERRORS = 91010

    # Subject Failures (92000 - 92990)
    FAILED_SUBJECT = 92000
    FAILED_SUBJECT_COMPLETE_WITH_ERRORS = 92010
    FAILED_SUBJECT_GROUP = 92020
    FAILED_SUBJECT_GROUP_COMPLETE_WITH_ERRORS = 92030

    # Iteration Failures (93000 - 93499)
    FAILED_ITERATION = 93000
    FAILED_ITERATION_COMPLETE_WITH_ERRORS = 93010

    # Pipeline Failures (94000 - 94999)
    FAILED_PIPELINE_COMPLETE_WITH_ERRORS = 94000
    FAILED_PIPELINE_EARLY_EXITED = 94010

    # Critical System Failures (98000 - 99999)
    FAILED_CRITICAL_SYSTEM_FAILURE = 98000

    def __str__(self):
        return self.name